// See license.txt for copyright information and redistribution information

#include "SDL.h"



// Some particle magic number definitions
#define IMMORTAL                -10                     // If lifespan=-10 it never decays
#define SPARKBASELIFE           5                       // Minimum spark lifespan
#define SPARKLIFERND            5                       // Lifetime random modifier
#define PARTICLE_DAMAGE         10                      // damage from particle collision

// Type of effect(s) to execute on collision with player object
#define GRAPPLE              0x10b                      // tether to player (usually -SPD too)
#define DO_DAMAGE            0x100b
#define PLUS_SCORE           0x10000b                   // add to player score
#define PLUS_HEALTH          0x100000b
#define NONE                 0

// type-defs for ai_mode, blit_method (globals.h)
typedef enum { stationary, weak_chase, strong_chase, weak_avoid, strong_avoid, circle_around,
    maintain_distance, no_AI } ai_modes;

typedef enum { sprite, alpha_add, alpha_sub, no_draw } blit_type;



// Linked list of basic particle objects
struct particle {
    struct particle *next;
    int id;
    float x, y;                           // (x,y) sub-pixel loc in game coordinates
    float vx, vy;                         // standard velocity vector components
    SDL_Rect loc;                         // game coordinates loc in pixels inc w,h
    SDL_Rect old_loc;                     // .... old position (still used? or debug?)
    blit_type blit_method;                // ADD, AVG, MUL, ... per-pixel blit type
    unsigned long int game_effect;                   // crash, bounce, +ENGINE, +SPEED, +FUEL, etc...
    int game_effect_var;                  // ... variable effect to apply: dmg, +fuel, etc.
    ai_modes current_ai_mode;            // AI routine to execute to define movement/decisions
    int lifespan;                         // lifespan measured in AI update ticks
    SDL_Surface *gfx;
};


#ifdef __cplusplus
extern "C" {
#endif

// Particle object functions
int particle_loadgfx( struct particle *mp, const char *filename );
void particle_draw( SDL_Surface *, struct particle * );
void spark_burst( struct particle *, int, int, float, float );
void spark_line( struct particle *, int, int, float, float );
void set_position( struct particle *, float, float );
void set_velocity( struct particle *, float, float );

// new, copy, delete particles
struct particle *new_particle( SDL_Surface*, int, ai_modes, unsigned long int, int, int );
struct particle *copy_particle( struct particle * );
// BROKEN void delete_particle( struct particle *previous, struct particle *p );

// Collide one particle list against another.
int particle_particle_collision( struct particle *, struct particle * );

#ifdef __cplusplus
}
#endif
