// See license.txt for copyright information and redistribution information

#define TRUE                1
#define FALSE               0

#define HIGHLIGHT           1
#define CLEAR               0

#define BUTTON_GFX          "button.bmp"
#define BUTTON_ACTIVE_GFX   "button_active.bmp"
#define TEXTBOX_GFX         "textbox.bmp"
#define PBOXFILL_GFX        "progress_bar_fill.bmp"



#ifdef __cplusplus
extern "C" {
#endif

// load a font just for the GUI

// rework gfx for slider bar
// refactor slider into generalized scrollbar, horz or vert

// click-to-position in text_box
// L and R arrow keys, DEL key

// list box with, optional scroll bar and buttons
// text field box: word wrap, dynamic size, optional scroll bar and buttons, smooth scroll and line fade





// all the random junk the top-level caller (usually main() in ANSI C) will
// need to do a simple gui
struct gui_manager {

    // linked lists of elements - contains first element of each type
    struct button *first_button;
    struct text_box *first_textbox;
    struct slider *first_slider;
    struct progressbar *progbar;
    struct text_block *first_textblock;

    unsigned int active_textbox;

    // min/max of window coords usable for gui drawing
    unsigned int winx0, winx1, winy0, winy1;

    // font size information
    unsigned int fonth, fontw;

    // gfx elements
    SDL_Surface *gui_surface;
    SDL_Surface *button_gfx, *button_active_gfx;
    SDL_Surface *textbox_gfx;
    SDL_Surface *pbarfill_gfx;
};

// progress bar display - unique item, only 1 on display at a time
struct progressbar {
    int x, y;
    int w, h;
    char text[ 256 ];
};

// typical button
struct button {
    int x, y;
    int w, h;
    char text[ 128 ];
    int id;
    struct button *next;
};

// text entry field with left-adjusted text label
struct text_box {
    int x, y;
    int w, h;
    int textwidth;          // # of characters to display in text box
    char text[ 256 ];       // label text
    char data[ 4096 ];      // data string storage
    int id;
    struct text_box *next;
};

// a scroll-able block of text
struct text_block {
    int x, y, w, h;
    int twidth;             // # of characters to store on each line, calc'ed from font size
    int theight;            // # of lines to display
    int start;              // the current line # at the top of the display
    int end;                // last line[] pointer that contains valid data
    char *line[ 256 ];      // 256 text lines, dynamic alloc'ed on struct initialization
    int id;
    struct text_block *next;
};

// grab-and-slide or click-to-position 0 to 100 integer slider
struct slider {
    int x, y;
    int w;
    int data;
    int id;
    struct slider *next;
};



// initialize the GUI manager structure
int init_gui( struct gui_manager * );

// initialize and add new items to linked list
int add_button( struct gui_manager *, int, int, int, int, char * );
int add_textbox( struct gui_manager *, int, int, int, int, char *, char * );
int add_slider( struct gui_manager *, int, int, int, int );
int add_textblock( struct gui_manager *, int, int, int, int );

// free dynamic subsets of GUI structs
void free_gui( struct gui_manager * );
void free_buttons( struct gui_manager * );
void free_textboxes( struct gui_manager * );
void free_sliders( struct gui_manager * );
void free_textblocks( struct gui_manager * );

// check if event occured (mouse) "in" GUI object
int check_buttons( struct gui_manager *, int, int );
int check_textboxes( struct gui_manager *, int, int );
int check_sliders( struct gui_manager *, int, int );

// GUI draw functions
void draw_gui( struct gui_manager * );
void draw_buttons( struct gui_manager * );
void draw_sliders( struct gui_manager * );
void draw_textboxes( struct gui_manager * );
void draw_textblocks( struct gui_manager * );
void highlight_button( struct gui_manager *, int, int );

// get pointer to object from the objects ID
struct button *get_button( struct gui_manager *, int );
struct text_box *get_textbox( struct gui_manager *, int );
struct text_block *get_textblock( struct gui_manager *, int );

// get last object in linked list
struct button *get_last_button( struct gui_manager * );
struct text_box *get_last_textbox( struct gui_manager * );
struct slider *get_last_slider( struct gui_manager * );
struct text_block *get_last_textblock( struct gui_manager * );


// NEW .. progress bar functions
void draw_progressbar( struct gui_manager *, float );
void init_progressbar( struct gui_manager *, int, int, int, int, char * );
void free_progressbar( struct gui_manager * );
void settext_progressbar( struct gui_manager *, char * );


// text entry fields
void draw_cursor( SDL_Surface *, struct text_box * );
void backspacechar( char * );
void addchar( char *, char );
//void mousepoint_cursor( struct text_box *, int, int );  // mouse point to a (new) cursor location
//void move_cursor( struct text_box *, int );  // L or R arrow keys to move

// text block functions
void clear_textblock( struct gui_manager *, int );  // clear dynamic, reset to initial conditions
void tb_addline( struct text_block *, char * );  // add a line to end of text block buffer
char *split_text( char *, char *, int );
void strip_newline( char * );
//void tb_scroll( struct text_block *, int );  // UP or DOWN, smooth scroll at pixel level
void tb_setposition( struct text_block *, int );  // set start line # to X
//void tb_fade( struct text block *, int );  // fade out entire text using timer callback
//void tb_fadeline( struct text_block *, int );  // fade out line # using timer callback

// sliders
//int check_sliders( struct slider *, int, int );
//struct slider *init_slider( struct slider *, int, int, int, int );
//void free_sliders( struct slider * );
#ifdef __cplusplus
}
#endif
